package clientapp;


import java.util.ArrayList;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import clientapp.view.AppointmentView;
import clientapp.view.ConsultView;
import clientapp.view.FamilyPhysicianView;
import clientapp.view.FindHomeServiceView;
import clientapp.view.HomeServiceView;
import clientapp.view.InformalCaregiver2View;
import clientapp.view.InformalCaregiverView;
import clientapp.view.MoveToHospitalView;
import clientapp.view.PCPhysicianView;
import clientapp.view.PatientRegistrationView;
import clientapp.view.PhysicianRegistrationView;
import clientapp.view.ReceiveReferralView;
import clientapp.view.WorklistView;


public class NavigationView extends ViewPart {
	public static final String ID = "clientapp.navigationView";
	
	public static final String WORKFLOW_INSTANCE = "Workflow Instance";
	public static final String WORKLIST = "WorkList";
	public static final String REGISTER_PHYSICIAN = "Register Physician";
	public static final String RECEIVE_REFERRAL = "Receive Referral";
	public static final String APPOINTMENT = "Appointment";
	public static final String CONSULTATION = "Consultation";
	public static final String INTAKE = "Intake";
	public static final String HOME_SERVICE = "Home Service";
	public static final String FIND_HOME_SERVICE = "Find Home Service";
	public static final String MOVE_SERVICE = "Move to Hospital";
	public static final String INFORMAL_CARE_GIVER = "Informal Caregiver";
	public static final String INFORMAL_CARE_GIVER2 = "Informal Caregiver2";
	public static final String FAMILY_PH = "Family Physician";
	public static final String PC_PH = "PC Physician";
	
	private TreeViewer viewer;
	 
	class TreeObject {
		private String name;
		private TreeParent parent;
		
		public TreeObject(String name) {
			this.name = name;
		}
		public String getName() {
			return name;
		}
		public void setParent(TreeParent parent) {
			this.parent = parent;
		}
		public TreeParent getParent() {
			return parent;
		}
		public String toString() {
			return getName();
		}
	}
	
	class TreeParent extends TreeObject {
		private ArrayList children;
		public TreeParent(String name) {
			super(name);
			children = new ArrayList();
		}
		public void addChild(TreeObject child) {
			children.add(child);
			child.setParent(this);
		}
		public void removeChild(TreeObject child) {
			children.remove(child);
			child.setParent(null);
		}
		public TreeObject[] getChildren() {
			return (TreeObject[]) children.toArray(new TreeObject[children.size()]);
		}
		public boolean hasChildren() {
			return children.size()>0;
		}
	}

	class ViewContentProvider implements IStructuredContentProvider, 
										   ITreeContentProvider {

		
        public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}
        
		public void dispose() {
		}
        
		public Object[] getElements(Object parent) {
			return getChildren(parent);
		}
        
		public Object getParent(Object child) {
			if (child instanceof TreeObject) {
				return ((TreeObject)child).getParent();
			}
			return null;
		}
        
		public Object[] getChildren(Object parent) {
			if (parent instanceof TreeParent) {
				return ((TreeParent)parent).getChildren();
			}
			return new Object[0];
		}

        public boolean hasChildren(Object parent) {
			if (parent instanceof TreeParent)
				return ((TreeParent)parent).hasChildren();
			return false;
		}
	}
	
	class ViewLabelProvider extends LabelProvider {

		private Shell shell;
		public ViewLabelProvider(Shell shell)
		{
			this.shell = shell;
		}
		public String getText(Object obj) {
			return obj.toString();
		}
		public Image getImage(Object obj) {
			System.out.println(obj);
			if(obj.toString().equals(WORKFLOW_INSTANCE))			
				return clientapp.Activator.getImageDescriptor("/icons/Settings16.jpg").createImage();
			
			else if(obj.toString().equals(HOME_SERVICE))
				return clientapp.Activator.getImageDescriptor("/icons/Home.jpg").createImage();
			
			else if(obj.toString().equals(WORKLIST))			
				return clientapp.Activator.getImageDescriptor("/icons/Cabinet.jpg").createImage();
						
			else if(obj.toString().equals(REGISTER_PHYSICIAN))
				return clientapp.Activator.getImageDescriptor("icons/doctor_icon.jpg").createImage();
			
			else if(obj.toString().equals(RECEIVE_REFERRAL))
				return clientapp.Activator.getImageDescriptor("/icons/Referral.jpg").createImage();
			
			else if(obj.toString().equals(APPOINTMENT))
				return clientapp.Activator.getImageDescriptor("/icons/appointmentB.gif").createImage();
			
			else if(obj.toString().equals(CONSULTATION))
				return clientapp.Activator.getImageDescriptor("/icons/consult_icon.png").createImage();
			
			else if(obj.toString().equals(INTAKE))
				return clientapp.Activator.getImageDescriptor("/icons/patient.gif").createImage();
			
			else if(obj.toString().equals(FIND_HOME_SERVICE))
				return clientapp.Activator.getImageDescriptor("/icons/home_search.gif").createImage();
			
			else if(obj.toString().equals(MOVE_SERVICE))
				return clientapp.Activator.getImageDescriptor("/icons/move.png").createImage();
			
			else if(obj.toString().equals(INFORMAL_CARE_GIVER))
				return clientapp.Activator.getImageDescriptor("/icons/icon_family_caregiver.gif").createImage();
			
			else if(obj.toString().equals(INFORMAL_CARE_GIVER2))
				return clientapp.Activator.getImageDescriptor("/icons/icon_family_caregiver.gif").createImage();
			
			else if(obj.toString().equals(FAMILY_PH))
				return clientapp.Activator.getImageDescriptor("/icons/family_ph.jpg").createImage();
			
			else if(obj.toString().equals(PC_PH))
				return clientapp.Activator.getImageDescriptor("/icons/pc_ph.gif").createImage();
			
			String imageKey = ISharedImages.IMG_OBJ_ELEMENT;
			if (obj instanceof TreeParent)
			   imageKey = ISharedImages.IMG_OBJ_FOLDER;
			return PlatformUI.getWorkbench().getSharedImages().getImage(imageKey);
		}
		
		
	}

    /**
     * We will set up a dummy model to initialize tree heararchy. In real
     * code, you will connect to a real model and expose its hierarchy.
     */
    private TreeObject createDummyModel() {
        TreeObject to1 = new TreeObject(RECEIVE_REFERRAL);
        TreeObject to2 = new TreeObject(APPOINTMENT);
        TreeObject to3 = new TreeObject(CONSULTATION);
        TreeObject to4 = new TreeObject(INTAKE);
        TreeParent p1 = new TreeParent("Overall");
        p1.addChild(to1);
        p1.addChild(to2);
        p1.addChild(to3);
        p1.addChild(to4);
        
        TreeObject toi = new TreeObject(FIND_HOME_SERVICE);
        TreeObject tom = new TreeObject(MOVE_SERVICE);
        TreeObject tocg = new TreeObject(INFORMAL_CARE_GIVER);
        TreeObject tocg2 = new TreeObject(INFORMAL_CARE_GIVER2);
        TreeObject tofp = new TreeObject(FAMILY_PH);
        TreeObject topp = new TreeObject(PC_PH);
        TreeParent p2 = new TreeParent("Team Building");        
        p2.addChild(toi);
        p2.addChild(tom);
        p2.addChild(tocg);
        p2.addChild(tocg2);
        p2.addChild(tofp);
        p2.addChild(topp);
        
        TreeObject tow = new TreeObject(WORKFLOW_INSTANCE);
        TreeObject towl = new TreeObject(WORKLIST);
        TreeObject top = new TreeObject(REGISTER_PHYSICIAN);
        TreeObject toh = new TreeObject(HOME_SERVICE);
        TreeParent p3 = new TreeParent("Resource Mgmt");
        p3.addChild(tow);
        p3.addChild(towl);
        p3.addChild(top);
        p3.addChild(toh);
        
        
        TreeParent root = new TreeParent("");
        root.addChild(p1);
        root.addChild(p2);
        root.addChild(p3);
        return root;
    }

	/**
     * This is a callback that will allow us to create the viewer and initialize
     * it.
     */
	public void createPartControl(final Composite parent) {
		viewer = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER);
		viewer.setContentProvider(new ViewContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider(parent.getShell()));
		viewer.setInput(createDummyModel());
		
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			
			
			public void selectionChanged(SelectionChangedEvent event) {
				
				TreeSelection selection = (TreeSelection) event.getSelection();
				try
				{
				if(selection.getFirstElement().toString().equals(WORKFLOW_INSTANCE))	
						getSite().getWorkbenchWindow().getActivePage().showView(WfInstanceView.ID, Integer.toString(WfInstanceView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);
			
				else if(selection.getFirstElement().toString().equals(HOME_SERVICE))				
					getSite().getWorkbenchWindow().getActivePage().showView(HomeServiceView.ID, Integer.toString(HomeServiceView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);
			
				else if(selection.getFirstElement().toString().equals(WORKLIST))				
						getSite().getWorkbenchWindow().getActivePage().showView(WorklistView.ID, Integer.toString(WorklistView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);
				
				else if(selection.getFirstElement().toString().equals(REGISTER_PHYSICIAN))
						getSite().getWorkbenchWindow().getActivePage().showView(PhysicianRegistrationView.ID, Integer.toString(PhysicianRegistrationView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);
				
				else if(selection.getFirstElement().toString().equals(RECEIVE_REFERRAL))									
						getSite().getWorkbenchWindow().getActivePage().showView(ReceiveReferralView.ID, Integer.toString(ReceiveReferralView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);									
				
				else if(selection.getFirstElement().toString().equals(APPOINTMENT))
					getSite().getWorkbenchWindow().getActivePage().showView(AppointmentView.ID, Integer.toString(AppointmentView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);
				
				else if(selection.getFirstElement().toString().equals(CONSULTATION))
					getSite().getWorkbenchWindow().getActivePage().showView(ConsultView.ID, Integer.toString(ConsultView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);
				
				else if(selection.getFirstElement().toString().equals(INTAKE))
					getSite().getWorkbenchWindow().getActivePage().showView(PatientRegistrationView.ID, Integer.toString(PatientRegistrationView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);
				
				else if(selection.getFirstElement().toString().equals(FIND_HOME_SERVICE))
					getSite().getWorkbenchWindow().getActivePage().showView(FindHomeServiceView.ID, Integer.toString(FindHomeServiceView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);
				
				else if(selection.getFirstElement().toString().equals(MOVE_SERVICE))
					getSite().getWorkbenchWindow().getActivePage().showView(MoveToHospitalView.ID, Integer.toString(MoveToHospitalView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);
				
				else if(selection.getFirstElement().toString().equals(INFORMAL_CARE_GIVER))
					getSite().getWorkbenchWindow().getActivePage().showView(InformalCaregiverView.ID, Integer.toString(InformalCaregiverView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);
				
				else if(selection.getFirstElement().toString().equals(INFORMAL_CARE_GIVER2))
					getSite().getWorkbenchWindow().getActivePage().showView(InformalCaregiver2View.ID, Integer.toString(InformalCaregiver2View.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);
				
				else if(selection.getFirstElement().toString().equals(FAMILY_PH))
					getSite().getWorkbenchWindow().getActivePage().showView(FamilyPhysicianView.ID, Integer.toString(FamilyPhysicianView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);
				
				else if(selection.getFirstElement().toString().equals(PC_PH))
					getSite().getWorkbenchWindow().getActivePage().showView(PCPhysicianView.ID, Integer.toString(PCPhysicianView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);
				
				} catch (PartInitException e) {
					MessageDialog.openError(parent.getShell(), "Error", "Error opening view:" + e.getMessage());
				}
			}
		});
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		viewer.getControl().setFocus();
	}
	
	
}


