package clientapp.view;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;

import pc.Overall.service.interfaces.IAppointmentService;
import pc.Overall.service.interfaces.IPhysicianService;
import pc.Overall.service.interfaces.IReceiveReferralService;
import pc.entity.AppointmentDTO;
import pc.entity.PhysicianDTO;
import pc.entity.ReferralInfoDTO;

import clientapp.IInstanceSelectionListener;

import ca.stfx.logic.cwf.engine.DateUtil;
import ca.stfx.logic.cwf.engine.base.AbstractTask;
import ca.stfx.logic.cwf.engine.base.entity.InstanceInfo;
import ca.stfx.logic.cwf.engine.base.entity.WfInstance;
import ca.stfx.logic.cwf.engine.service.IWorkflowEngineService;

import communicator.RMIServiceProvider;

public class AppointmentView extends ViewPart implements IInstanceSelectionListener{

	private List<WfInstance> activeInstances;
	public static final String TASK_ID = "Overall_Appointment";
	public static final String TASK_NAME = "Appointment";
	
	public static final String ID = "clientapp.AppointmentView";
	public static int instanceNumber = 0;
	
	private Button button;
	private Button phButton;
	private Combo combo;
	private Shell shell;
	
	private Text txtPatientName;
	private Text txtPhysicianId;
	private Text txtPhysicianName;
	private Text txtPhysicianDesignation;
	private Text txtAppointmentDate;
	private Text txtAppointmentTime;	
	private Text txtNarrative;
	
	private long selectedInstanceId = 0;
	
	public void createPartControl(final Composite parent) {
		this.shell = parent.getShell();
		selectedInstanceId = -1;
		Composite top = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		top.setLayout(layout);
		// top banner
		Composite banner = new Composite(top, SWT.NONE);
		banner.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL, GridData.VERTICAL_ALIGN_BEGINNING, true, false));
		layout = new GridLayout();
		layout.marginHeight = 5;
		layout.marginWidth = 10;
		layout.numColumns = 1;
		banner.setLayout(layout);
		
		// setup bold font
		Font boldFont = JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT);    
		
		Label l = new Label(banner, SWT.WRAP);
		l.setText("Appointment for patient");
		l.setFont(boldFont);
		
		Calendar theCal = Calendar.getInstance();
		String dateStr = "Date: " + theCal.get(Calendar.DATE) + "/" + theCal.get(Calendar.MONTH) + "/" + theCal.get(Calendar.YEAR);
		
		l = new Label(banner, SWT.WRAP);
		l.setText(dateStr);
		
			
		Composite body = new Composite(top, SWT.NONE);
		body.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL, GridData.VERTICAL_ALIGN_BEGINNING, true, false));
		layout = new GridLayout();
		layout.marginHeight = 5;
		layout.marginWidth = 10;
		layout.numColumns = 2;
		body.setLayout(layout);
		
		populateActiveInstances();
		
		l = new Label(body, SWT.WRAP);
		l.setText("Active Instance:");
		combo = new Combo(body, SWT.DROP_DOWN | SWT.BORDER | SWT.READ_ONLY);
		for(WfInstance instance : activeInstances)
		{
			combo.add("Instance #" + instance.getId());
		}
		
	    combo.addSelectionListener(new SelectionListener() {
			
			public void widgetSelected(SelectionEvent e) {
				if(e == null)
					return;			
				WfInstance selectedInstance = getInstanceByText(((Combo) e.getSource()).getText());
				processSelection(selectedInstance.getId());
			}
			
			public void widgetDefaultSelected(SelectionEvent e) {
				// TODO Auto-generated method stub
				
			}
		});
	    
		l = new Label(body, SWT.WRAP);
		l.setText("Patient Name:");
		txtPatientName = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtPatientName.setText("");
		txtPatientName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		txtPatientName.setEditable(false);
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Physician Id:");
		txtPhysicianId = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtPhysicianId.setText("");
		txtPhysicianId.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		
		l = new Label(body,SWT.WRAP);
		l.setText("");
		
		phButton = new Button(body, SWT.PUSH);
		phButton.setText("FIND");		
		phButton.addMouseListener(new MouseListener() {			
			public void mouseUp(MouseEvent e) {
			}
			public void mouseDown(MouseEvent e) {				
				if( validateField(txtPhysicianId, "Physician Id") == false)
					return;				
				
				// invoke the server
				Long physicianId = Long.parseLong(txtPhysicianId.getText());
				
				IPhysicianService phService = RMIServiceProvider.getRMIServiceProvider().getPhysicianService();
				try
				{
					PhysicianDTO physician = phService.findPhysicianInfoById(physicianId);
					if(physician == null)
						MessageDialog.openError(shell, "Error", "Physician Id :" + physicianId + " does not exists.");
					else
					{
						txtPhysicianName.setText(physician.getLastName() + ", " + physician.getFirstName() );
						txtPhysicianDesignation.setText(physician.getDesignation());
					}
					
				}
				catch(Exception ex)
				{
					MessageDialog.openError(shell, "Error", ex.getMessage());
				}
			}
			public void mouseDoubleClick(MouseEvent e) {
			}
		});
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Physician Name:");
		txtPhysicianName = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtPhysicianName.setText("");
		txtPhysicianName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		txtPhysicianName.setEditable(false);
		
		l = new Label(body, SWT.WRAP);
		l.setText("Designation:");
		txtPhysicianDesignation = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtPhysicianDesignation.setText("");
		txtPhysicianDesignation.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		txtPhysicianDesignation.setEditable(false);
		
		l = new Label(body, SWT.WRAP);
		l.setText("Appointment Date:");
		txtAppointmentDate = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtAppointmentDate.setText("");
		txtAppointmentDate.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Appointment Time:");
		txtAppointmentTime = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtAppointmentTime.setText("");
		txtAppointmentTime.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		l = new Label(top, SWT.WRAP);
		l.setText("Narrative:");
		txtNarrative = new Text(top, SWT.MULTI | SWT.WRAP);
		txtNarrative.setText("");
		txtNarrative.setLayoutData(new GridData(GridData.FILL_BOTH));
		

		button = new Button(top, SWT.PUSH);
		button.setText("ACTION");
		button.setEnabled(false);
		button.addMouseListener(new MouseListener() {			
			public void mouseUp(MouseEvent e) {
			}
			public void mouseDown(MouseEvent e) {
				if(selectedInstanceId ==-1)
				{
					MessageDialog.openError(shell, "Error",  "You have not selected any instance");
					return ;
				}
				if( validateField(txtPhysicianId, "Physician Id") == false)
					return;
				if( validateField(txtAppointmentDate, "Appointment Date") == false)
					return;
				if( validateField(txtAppointmentTime, "Appointment Time") == false)
					return;
				
				// invoke the server
				AppointmentDTO appointment = new AppointmentDTO();
				appointment.setAppointmentDate(txtAppointmentDate.getText());
				appointment.setAppointmentTime(txtAppointmentTime.getText());
				appointment.setCreationDate(DateUtil.getCurrentDate());
				appointment.setInstanceId(selectedInstanceId);
				appointment.setNarrative(txtNarrative.getText());
				appointment.setPatientName(txtPatientName.getText());
				appointment.setPhysicianId(Long.parseLong(txtPhysicianId.getText()));
				
				IAppointmentService appointmentService = RMIServiceProvider.getRMIServiceProvider().getAppointmentService();
				try
				{
					appointment = appointmentService.saveAppointment(appointment);
					MessageDialog.openInformation(parent.getShell(), "Information", "Appointment " + appointment.getId() + " created successfully.");
					button.setEnabled(false);
				}
				catch(Exception ex)
				{
					MessageDialog.openError(shell, "Error", ex.getMessage());
				}
			}
			public void mouseDoubleClick(MouseEvent e) {
			}
		});

	}
	
	private boolean validateField(Text txtField, String fieldName)
	{
		if(txtField.getText().length() == 0){
			MessageDialog.openError(shell, "Error",  fieldName + " cannot be empty.");
			return false;
		}
		return true;
	}
	
	public void setActiveInstance(Long instanceId)
	{
		combo.setText( "Instance #" + instanceId);
		processSelection(instanceId);
	}
	
	
	private void processSelection(Long instanceId)
	{		
		this.selectedInstanceId = instanceId;
		IWorkflowEngineService wfEngineService = RMIServiceProvider.getRMIServiceProvider().getWorkflowEngineService();
		List<InstanceInfo> infos = wfEngineService.getAvailableMethods(instanceId, TASK_ID);
		button.setEnabled(false);
		boolean flag = false;
		
		for(InstanceInfo info : infos)
		{
			if(info.getAvailableMethod().equals(AbstractTask._ACTION)){				
					flag = true;
					break;			
			}
		}
		if(flag == false)
			return;
		
		button.setEnabled(true);
		// find the referral by instance id and get the patient name
		IReceiveReferralService receiveReferralService = RMIServiceProvider.getRMIServiceProvider().getReceiveReferralService();
		ReferralInfoDTO referral = receiveReferralService.findReferralInfoByInstanceId(instanceId);
		txtPatientName.setText(referral.getPatientName());
		
	}
	
	private void populateActiveInstances()
	{
		IWorkflowEngineService wfEngineService = RMIServiceProvider.getRMIServiceProvider().getWorkflowEngineService();
		activeInstances = wfEngineService.getAllActiveInstances();
		if(activeInstances == null)
			activeInstances = new ArrayList<WfInstance>();
	}
	private WfInstance getInstanceByText(String instanceText)
	{
		for(WfInstance ins : activeInstances)
		{
			String name = "Instance #" + ins.getId();
			if(name.equals(instanceText))
				return ins;
		}
		return null;
	}

	public void setFocus() {
	}
}
