package clientapp.view;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;

import pc.Overall.service.interfaces.IAppointmentService;
import pc.Overall.service.interfaces.IPatientRegistrationService;
import pc.Overall.service.interfaces.IPhysicianService;
import pc.Overall.service.interfaces.IReceiveReferralService;
import pc.TeamBuilding.service.interfaces.IHomeService;
import pc.entity.AppointmentDTO;
import pc.entity.HomeServiceDTO;
import pc.entity.PatientDTO;
import pc.entity.PhysicianDTO;
import pc.entity.ReferralInfoDTO;

import clientapp.IInstanceSelectionListener;

import ca.stfx.logic.cwf.engine.DateUtil;
import ca.stfx.logic.cwf.engine.base.AbstractTask;
import ca.stfx.logic.cwf.engine.base.entity.InstanceInfo;
import ca.stfx.logic.cwf.engine.base.entity.WfInstance;
import ca.stfx.logic.cwf.engine.service.IWorkflowEngineService;

import communicator.RMIServiceProvider;

public class FindHomeServiceView extends ViewPart implements IInstanceSelectionListener{

	private List<WfInstance> activeInstances;
	public static final String TASK_ID = "Team_Building_Find_HomeService";
	public static final String TASK_NAME = "Find_HomeService";
	
	public static final String ID = "clientapp.FindHomeServiceView";
	public static int instanceNumber = 0;
	
	private Button actionButton;
	private Button abortButton;
	private Button failButton;
	private Button findButton;
	private Combo combo;
	private Shell shell;
	
	private Text txtPatientName;
	private Text txtZip;		
	private Text txtContactPerson;
	private Text txtPhone;
	private Table table;
	
	private List<HomeServiceDTO> homeServices;
	private HomeServiceDTO selectedHomeService;	
	private long selectedInstanceId = 0;
	
	public void createPartControl(final Composite parent) {
		this.shell = parent.getShell();
		selectedInstanceId = -1;
		Composite top = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		top.setLayout(layout);
		// top banner
		Composite banner = new Composite(top, SWT.NONE);
		banner.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL, GridData.VERTICAL_ALIGN_BEGINNING, true, false));
		layout = new GridLayout();
		layout.marginHeight = 5;
		layout.marginWidth = 10;
		layout.numColumns = 1;
		banner.setLayout(layout);
		
		// setup bold font
		Font boldFont = JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT);    
		
		Label l = new Label(banner, SWT.WRAP);
		l.setText("Find Home Service for patient");
		l.setFont(boldFont);
		
		Calendar theCal = Calendar.getInstance();
		String dateStr = "Date: " + theCal.get(Calendar.DATE) + "/" + theCal.get(Calendar.MONTH) + "/" + theCal.get(Calendar.YEAR);
		
		l = new Label(banner, SWT.WRAP);
		l.setText(dateStr);
		
			
		Composite body = new Composite(top, SWT.NONE);
		body.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL, GridData.VERTICAL_ALIGN_BEGINNING, true, false));
		layout = new GridLayout();
		layout.marginHeight = 5;
		layout.marginWidth = 10;
		layout.numColumns = 2;
		body.setLayout(layout);
		
		populateActiveInstances();
		
		l = new Label(body, SWT.WRAP);
		l.setText("Active Instance:");
		combo = new Combo(body, SWT.DROP_DOWN | SWT.BORDER | SWT.READ_ONLY);
		for(WfInstance instance : activeInstances)
		{
			combo.add("Instance #" + instance.getId());
		}
		
	    combo.addSelectionListener(new SelectionListener() {
			
			public void widgetSelected(SelectionEvent e) {
				if(e == null)
					return;			
				WfInstance selectedInstance = getInstanceByText(((Combo) e.getSource()).getText());
				processSelection(selectedInstance.getId());
			}
			
			public void widgetDefaultSelected(SelectionEvent e) {
				// TODO Auto-generated method stub
				
			}
		});
	    
		l = new Label(body, SWT.WRAP);
		l.setText("Patient Name:");
		txtPatientName = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtPatientName.setText("");
		txtPatientName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		txtPatientName.setEditable(false);
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Zip:");
		txtZip = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtZip.setText("");
		txtZip.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		
		
		l = new Label(body,SWT.WRAP);
		l.setText("");
		
		findButton = new Button(body, SWT.PUSH);
		findButton.setText("FIND");		
		findButton.addMouseListener(new MouseListener() {			
			public void mouseUp(MouseEvent e) {
			}
			public void mouseDown(MouseEvent e) {				
				if( validateField(txtZip, "Zip Code") == false)
					return;				
				
				// invoke the server
				String zip = txtZip.getText();
				
				IHomeService homeService = RMIServiceProvider.getRMIServiceProvider().getHomeService();
				try
				{
					List<HomeServiceDTO> hsInfos = homeService.findHomeServiceByZipCode(zip);
					//populate the table
					fillTable(table, hsInfos);
				}
				catch(Exception ex)
				{
					MessageDialog.openError(shell, "Error", ex.getMessage());
				}
			}
			public void mouseDoubleClick(MouseEvent e) {
			}
		});
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Contact Person:");
		txtContactPerson = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtContactPerson.setText("");
		txtContactPerson.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		txtContactPerson.setEditable(false);
		
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Phone:");
		txtPhone = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtPhone.setText("");
		txtPhone.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		txtPhone.setEditable(false);
		
		
		table = new Table(top, SWT.NONE);
	    table.setHeaderVisible(true);
	    table.setLinesVisible(true);
	    TableColumn[] column = new TableColumn[4];
	    column[0] = new TableColumn(table, SWT.NONE);
	    column[0].setText("Index");
	    
	    column[1] = new TableColumn(table, SWT.NONE);
	    column[1].setText("Contact Person");
	    
	    column[2] = new TableColumn(table, SWT.NONE);
	    column[2].setText("Address1");
	    
	    column[3] = new TableColumn(table, SWT.RIGHT);
	    column[3].setText("Address2");
	    
	    for (int i = 0, n = column.length; i < n; i++) {
	      column[i].pack();
	    }
	    table.setLayoutData(new GridData(GridData.FILL_BOTH));
	    
	    table.addListener(SWT.Selection, new Listener() {
	        public void handleEvent(Event e) {
	          TableItem[] selection = table.getSelection();
	          if(selection == null || selection.length == 0 )
	        	  return;
	          String indexNo = selection[0].getText();
	          System.out.println("Selected table entry : " + indexNo);
	          int index = Integer.parseInt(indexNo);
	          index--;
	          HomeServiceDTO selectedEntry = homeServices.get(index);
	          txtContactPerson.setText(selectedEntry.getContactPerson());
	          txtPhone.setText(selectedEntry.getPhoneNumber());
	          selectedHomeService = selectedEntry;
	        }
	      });
		
		
		Composite buttonPane = new Composite(top, SWT.NONE);
		buttonPane.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL, GridData.VERTICAL_ALIGN_BEGINNING, true, false));
		layout = new GridLayout();
		layout.marginHeight = 5;
		layout.marginWidth = 10;
		layout.numColumns = 3;
		buttonPane.setLayout(layout);		

		actionButton = new Button(buttonPane, SWT.PUSH);
		actionButton.setText("ACTION");
		actionButton.setEnabled(false);
		actionButton.addMouseListener(new MouseListener() {			
			public void mouseUp(MouseEvent e) {
			}
			public void mouseDown(MouseEvent e) {
				if(selectedInstanceId ==-1)
				{
					MessageDialog.openError(shell, "Error",  "You have not selected any instance");
					return ;
				}
				if( validateField(txtContactPerson, "Contact Person") == false)
					return;
				if( validateField(txtPhone, "Phone Number") == false)
					return;
				
				try
				{
					// invoke the server
					IPatientRegistrationService patientService = RMIServiceProvider.getRMIServiceProvider().getPatientRegistrationService();
					PatientDTO patient = patientService.findPatientInfoByInstanceId(selectedInstanceId);
					
					IHomeService homeService = RMIServiceProvider.getRMIServiceProvider().getHomeService();
					homeService.allocateHomeServiceToPatient(patient, selectedHomeService);
					
					MessageDialog.openInformation(parent.getShell(), "Information", "Home service assigned successfully.");
					actionButton.setEnabled(false);
					abortButton.setEnabled(false);
					failButton.setEnabled(false);
				}
				catch(Exception ex)
				{
					MessageDialog.openError(shell, "Error", ex.getMessage());
				}
			}
			public void mouseDoubleClick(MouseEvent e) {
			}
		});
		
		abortButton = new Button(buttonPane, SWT.PUSH);
		abortButton.setText("ABORT");
		abortButton.setEnabled(false);
		abortButton.addMouseListener(new MouseListener() {			
			public void mouseUp(MouseEvent e) {
			}
			public void mouseDown(MouseEvent e) {
				if(selectedInstanceId ==-1)
				{
					MessageDialog.openError(shell, "Error",  "You have not selected any instance");
					return ;
				}
								
				try
				{
				
					IHomeService homeService = RMIServiceProvider.getRMIServiceProvider().getHomeService();
					homeService.abort(selectedInstanceId);
					
					MessageDialog.openInformation(parent.getShell(), "Information", "Operation aborted successfully.");
					actionButton.setEnabled(false);
					abortButton.setEnabled(false);
					failButton.setEnabled(false);
				}
				catch(Exception ex)
				{
					MessageDialog.openError(shell, "Error", ex.getMessage());
				}
			}
			public void mouseDoubleClick(MouseEvent e) {
			}
		});		

		
		failButton = new Button(buttonPane, SWT.PUSH);
		failButton.setText("FAIL");
		failButton.setEnabled(false);
		failButton.addMouseListener(new MouseListener() {			
			public void mouseUp(MouseEvent e) {
			}
			public void mouseDown(MouseEvent e) {
				if(selectedInstanceId ==-1)
				{
					MessageDialog.openError(shell, "Error",  "You have not selected any instance");
					return ;
				}
								
				try
				{
				
					IHomeService homeService = RMIServiceProvider.getRMIServiceProvider().getHomeService();
					homeService.fail(selectedInstanceId);
					
					MessageDialog.openInformation(parent.getShell(), "Information", "Operation aborted successfully.");
					actionButton.setEnabled(false);
					abortButton.setEnabled(false);
					failButton.setEnabled(false);
				}
				catch(Exception ex)
				{
					MessageDialog.openError(shell, "Error", ex.getMessage());
				}
			}
			public void mouseDoubleClick(MouseEvent e) {
			}
		});		
	}
	
	 private void fillTable(final Table table, List<HomeServiceDTO> homeServices) {
		    table.setRedraw(false);
		    if(homeServices == null)
		    	return;
		    
		    this.homeServices = homeServices;
		    table.removeAll();
		    
		    for(int i = 0; i < homeServices.size(); ++i){
		      HomeServiceDTO theHomeService = homeServices.get(i);
		      TableItem item = new TableItem(table, SWT.NONE);
		      int c = 0;
		      item.setText(c++, ""+(i+1) );
		      item.setText(c++, theHomeService.getContactPerson());
		      item.setText(c++, theHomeService.getAddress1());
		      item.setText(c++, theHomeService.getAddress2());
		            
		    }
		    table.setLayoutData(new GridData(GridData.FILL_BOTH));
		    table.setRedraw(true);		    		    
		  }
	
	
	private boolean validateField(Text txtField, String fieldName)
	{
		if(txtField.getText().length() == 0){
			MessageDialog.openError(shell, "Error",  fieldName + " cannot be empty.");
			return false;
		}
		return true;
	}
	
	public void setActiveInstance(Long instanceId)
	{
		combo.setText( "Instance #" + instanceId);
		processSelection(instanceId);
	}
	
	
	
	private void processSelection(Long instanceId)
	{		
		this.selectedInstanceId = instanceId;
		IWorkflowEngineService wfEngineService = RMIServiceProvider.getRMIServiceProvider().getWorkflowEngineService();
		List<InstanceInfo> infos = wfEngineService.getAvailableMethods(instanceId, TASK_ID);
		
		actionButton.setEnabled(false);
		abortButton.setEnabled(false);
		failButton.setEnabled(false);
		
		for(InstanceInfo info : infos)
		{
			if(info.getAvailableMethod().equals(AbstractTask._ACTION))				
				actionButton.setEnabled(true);
			if(info.getAvailableMethod().equals(AbstractTask._ABORT))				
				abortButton.setEnabled(true);
			if(info.getAvailableMethod().equals(AbstractTask._FAIL))				
				failButton.setEnabled(true);
			
		}
		
		// find the patien by instance id and get the patient name
		IPatientRegistrationService patientService = RMIServiceProvider.getRMIServiceProvider().getPatientRegistrationService();
		PatientDTO patient = patientService.findPatientInfoByInstanceId(instanceId);
		txtPatientName.setText(patient.getLastName() + ", " + patient.getFirstName());
		txtZip.setText(patient.getZip());
		
		
	}
	
	private void populateActiveInstances()
	{
		IWorkflowEngineService wfEngineService = RMIServiceProvider.getRMIServiceProvider().getWorkflowEngineService();
		activeInstances = wfEngineService.getAllActiveInstances();
		if(activeInstances == null)
			activeInstances = new ArrayList<WfInstance>();
	}
	private WfInstance getInstanceByText(String instanceText)
	{
		for(WfInstance ins : activeInstances)
		{
			String name = "Instance #" + ins.getId();
			if(name.equals(instanceText))
				return ins;
		}
		return null;
	}

	public void setFocus() {
	}
}
