package clientapp.view;



import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;

import pc.TeamBuilding.service.interfaces.IHomeService;
import pc.entity.HomeServiceDTO;
import communicator.RMIServiceProvider;
import ca.stfx.logic.cwf.engine.DateUtil;



public class HomeServiceView extends ViewPart {

	public static final String ID = "clientapp.HomeServiceView";
	public static int instanceNumber = 0;
	
	private Text txtContactPerson;
	private Text txtAddress1;
	private Text txtAddress2;
	private Text txtPhoneNumber;	
	private Text txtWorkingArea;
	private Shell shell;
	Composite top; 
	
	public void createPartControl(final Composite parent) {
		this.top = new Composite(parent, SWT.NONE);
		this.shell = parent.getShell();
		GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		top.setLayout(layout);
		// top banner
		Composite banner = new Composite(top, SWT.NONE);
		banner.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL, GridData.VERTICAL_ALIGN_BEGINNING, true, false));
		layout = new GridLayout();
		layout.marginHeight = 5;
		layout.marginWidth = 10;
		layout.numColumns = 1;
		banner.setLayout(layout);
		
		// setup bold font
		Font boldFont = JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT);    
		
		Label l = new Label(banner, SWT.WRAP);
		l.setText("Select an instance from the drop down.\nAvailable workitems will be displayed in the list");
		l.setFont(boldFont);
						
		Composite body = new Composite(top, SWT.NONE);
		body.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL, GridData.VERTICAL_ALIGN_BEGINNING, true, false));
		layout = new GridLayout();
		layout.marginHeight = 5;
		layout.marginWidth = 10;
		layout.numColumns = 2;
		body.setLayout(layout);
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Contact Person:");
		txtContactPerson = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtContactPerson.setText("");
		txtContactPerson.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
	    		
		l = new Label(body, SWT.WRAP);
		l.setText("Address1:");
		txtAddress1 = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtAddress1.setText("");
		txtAddress1.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		l = new Label(body, SWT.WRAP);
		l.setText("Address2:");
		txtAddress2 = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtAddress2.setText("");
		txtAddress2.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		l = new Label(body, SWT.WRAP);
		l.setText("Phone Number:");
		txtPhoneNumber = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtPhoneNumber.setText("");
		txtPhoneNumber.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Working Area:");
		txtWorkingArea = new Text(top, SWT.MULTI | SWT.WRAP);
		txtWorkingArea.setText("");
		txtWorkingArea.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		Button button = new Button(top, SWT.PUSH);
		button.setText("ADD");
		
		button.addMouseListener(new MouseListener() {			
			public void mouseUp(MouseEvent e) {
			}
			public void mouseDown(MouseEvent e) {
				// invoke the server
				if( validateField(txtContactPerson, "Contact Person") == false)
					return;
				if( validateField(txtPhoneNumber, "Phone Number") == false)
					return;
				
				HomeServiceDTO homeService = new HomeServiceDTO();
				homeService.setAddress1(txtAddress1.getText());
				homeService.setAddress2(txtAddress2.getText());
				homeService.setContactPerson(txtContactPerson.getText());
				homeService.setCreationDate(DateUtil.getCurrentDate());
				homeService.setPhoneNumber(txtPhoneNumber.getText());
				homeService.setWorkingArea(txtWorkingArea.getText());
				
				IHomeService homeServiceS = RMIServiceProvider.getRMIServiceProvider().getHomeService();
				try
				{
					homeService = homeServiceS.saveHomeService(homeService); 
					MessageDialog.openInformation(parent.getShell(), "Information", "Home Service Id " + homeService.getId() + " created successfully.");					
				}
				catch(Exception ex)
				{
					MessageDialog.openError(shell, "Error", ex.getMessage());
				}
			}
			public void mouseDoubleClick(MouseEvent e) {
			}
		});
		
	}

	
	private boolean validateField(Text txtField, String fieldName)
	{
		if(txtField.getText().length() == 0){
			MessageDialog.openError(shell, "Error",  fieldName + " cannot be empty.");
			return false;
		}
		return true;
	}
	
	public void setFocus() {
	}
}
