package clientapp.view;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;

import pc.Overall.service.interfaces.IAppointmentService;
import pc.Overall.service.interfaces.IPatientRegistrationService;
import pc.Overall.service.interfaces.IPhysicianService;
import pc.Overall.service.interfaces.IReceiveReferralService;
import pc.TeamBuilding.service.interfaces.IHomeService;
import pc.TeamBuilding.service.interfaces.IInformalCaregiverService;
import pc.TeamBuilding.service.interfaces.IMoveToHospitalService;
import pc.entity.AppointmentDTO;
import pc.entity.InformalCaregiverInfoDTO;
import pc.entity.PatientDTO;
import pc.entity.PhysicianDTO;
import pc.entity.ReferralInfoDTO;

import clientapp.IInstanceSelectionListener;

import ca.stfx.logic.cwf.engine.DateUtil;
import ca.stfx.logic.cwf.engine.base.AbstractTask;
import ca.stfx.logic.cwf.engine.base.entity.InstanceInfo;
import ca.stfx.logic.cwf.engine.base.entity.WfInstance;
import ca.stfx.logic.cwf.engine.service.IWorkflowEngineService;

import communicator.RMIServiceProvider;

public class InformalCaregiver2View extends ViewPart implements IInstanceSelectionListener{

	private List<WfInstance> activeInstances;
	public static final String TASK_ID = "Team_Building_Informal_CareGiver2";
	public static final String TASK_NAME = "Informal_CareGiver2";
	
	public static final String ID = "clientapp.InformalCaregiver2View";
	public static int instanceNumber = 0;
	
	private Button actionButton;
	private Button abortButton;
	private Button failButton;
	
	private Combo combo;
	private Shell shell;
	
	private Text txtPatientName;
	private Text txtPatientId;
	
	private Text txtLastName;
	private Text txtFirstName;
	private Text txtAddress1;
	private Text txtAddress2;
	private Text txtPhone;
	private Text txtZip;
	private Text txtRelationship;	
	private Text txtNarrative;
	
		
	private long selectedInstanceId = 0;
	
	public void createPartControl(final Composite parent) {
		this.shell = parent.getShell();
		selectedInstanceId = -1;
		Composite top = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		top.setLayout(layout);
		// top banner
		Composite banner = new Composite(top, SWT.NONE);
		banner.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL, GridData.VERTICAL_ALIGN_BEGINNING, true, false));
		layout = new GridLayout();
		layout.marginHeight = 5;
		layout.marginWidth = 10;
		layout.numColumns = 1;
		banner.setLayout(layout);
		
		// setup bold font
		Font boldFont = JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT);    
		
		Label l = new Label(banner, SWT.WRAP);
		l.setText("Appointment for patient");
		l.setFont(boldFont);
		
		Calendar theCal = Calendar.getInstance();
		String dateStr = "Date: " + theCal.get(Calendar.DATE) + "/" + theCal.get(Calendar.MONTH) + "/" + theCal.get(Calendar.YEAR);
		
		l = new Label(banner, SWT.WRAP);
		l.setText(dateStr);
		
			
		Composite body = new Composite(top, SWT.NONE);
		body.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL, GridData.VERTICAL_ALIGN_BEGINNING, true, false));
		layout = new GridLayout();
		layout.marginHeight = 5;
		layout.marginWidth = 10;
		layout.numColumns = 2;
		body.setLayout(layout);
		
		populateActiveInstances();
		
		l = new Label(body, SWT.WRAP);
		l.setText("Active Instance:");
		combo = new Combo(body, SWT.DROP_DOWN | SWT.BORDER | SWT.READ_ONLY);
		for(WfInstance instance : activeInstances)
		{
			combo.add("Instance #" + instance.getId());
		}
		
	    combo.addSelectionListener(new SelectionListener() {
			
			public void widgetSelected(SelectionEvent e) {
				if(e == null)
					return;			
				WfInstance selectedInstance = getInstanceByText(((Combo) e.getSource()).getText());
				processSelection(selectedInstance.getId());
			}
			
			public void widgetDefaultSelected(SelectionEvent e) {
				// TODO Auto-generated method stub
				
			}
		});
	    
		l = new Label(body, SWT.WRAP);
		l.setText("Patient Name:");
		txtPatientName = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtPatientName.setText("");
		txtPatientName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		txtPatientName.setEditable(false);
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Patient Id:");
		txtPatientId = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtPatientId.setText("");
		txtPatientId.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		txtPatientId.setEditable(false);

		
	
		
		l = new Label(body, SWT.WRAP);
		l.setText("Last Name:");
		txtLastName = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtLastName.setText("");
		txtLastName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("First Name:");
		txtFirstName = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtFirstName.setText("");
		txtFirstName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		l = new Label(body, SWT.WRAP);
		l.setText("Address1:");
		txtAddress1 = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtAddress1.setText("");
		txtAddress1.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Address2:");
		txtAddress2 = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtAddress2.setText("");
		txtAddress2.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Phone:");
		txtPhone = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtPhone.setText("");
		txtPhone.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Zip:");
		txtZip = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtZip.setText("");
		txtZip.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Relationship:");
		txtRelationship = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtRelationship.setText("");
		txtRelationship.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		
		l = new Label(top, SWT.WRAP);
		l.setText("Narrative:");
		txtNarrative = new Text(top, SWT.MULTI | SWT.WRAP);
		txtNarrative.setText("");
		txtNarrative.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		
		Composite buttonPane = new Composite(top, SWT.NONE);
		buttonPane.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL, GridData.VERTICAL_ALIGN_BEGINNING, true, false));
		layout = new GridLayout();
		layout.marginHeight = 5;
		layout.marginWidth = 10;
		layout.numColumns = 3;
		buttonPane.setLayout(layout);
		
		actionButton = new Button(buttonPane, SWT.PUSH);
		actionButton.setText("ACTION");
		actionButton.setEnabled(false);
		actionButton.addMouseListener(new MouseListener() {			
			public void mouseUp(MouseEvent e) {
			}
			public void mouseDown(MouseEvent e) {
				if(selectedInstanceId ==-1)
				{
					MessageDialog.openError(shell, "Error",  "You have not selected any instance");
					return ;
				}
				if( validateField(txtPatientName, "Patient Name") == false)
					return;
				if( validateField(txtLastName, "Last Name") == false)
					return;
				if( validateField(txtFirstName, "First Name") == false)
					return;
				if( validateField(txtPhone, "Phone Number") == false)
					return;
				if( validateField(txtRelationship, "Relationship") == false)
					return;				
				try
				{
					InformalCaregiverInfoDTO icInfo = new InformalCaregiverInfoDTO();
					icInfo.setAddress1(txtAddress1.getText());
					icInfo.setAddress2(txtAddress2.getText());
					icInfo.setCreationDate(DateUtil.getCurrentDate());
					icInfo.setFirstName(txtFirstName.getText());
					icInfo.setInstanceId(selectedInstanceId);
					icInfo.setLastName(txtLastName.getText());
					icInfo.setNarrative(txtNarrative.getText());
					icInfo.setPatientId(Long.parseLong(txtPatientId.getText()));
					icInfo.setPhone(txtPhone.getText());
					icInfo.setRelationship(txtRelationship.getText());
					icInfo.setZip(txtZip.getText());
					
					IInformalCaregiverService icgService = RMIServiceProvider.getRMIServiceProvider().getInformalCaregiver2Service();
					icInfo = icgService.saveInformalCaregiverInfo(icInfo);
					
					MessageDialog.openInformation(parent.getShell(), "Information", "Informal Caregiver " + icInfo.getId() + " successfully.");
					actionButton.setEnabled(false);
					abortButton.setEnabled(false);
					failButton.setEnabled(false);
				}
				catch(Exception ex)
				{
					MessageDialog.openError(shell, "Error", ex.getMessage());
				}
			}
			public void mouseDoubleClick(MouseEvent e) {
			}
		});
		
		abortButton = new Button(buttonPane, SWT.PUSH);
		abortButton.setText("ABORT");
		abortButton.setEnabled(false);
		abortButton.addMouseListener(new MouseListener() {			
			public void mouseUp(MouseEvent e) {
			}
			public void mouseDown(MouseEvent e) {
				if(selectedInstanceId ==-1)
				{
					MessageDialog.openError(shell, "Error",  "You have not selected any instance");
					return ;
				}
								
				try
				{
					IInformalCaregiverService icgService = RMIServiceProvider.getRMIServiceProvider().getInformalCaregiver2Service();
					icgService.abort(selectedInstanceId);
					
					MessageDialog.openInformation(parent.getShell(), "Information", "Operation aborted successfully.");
					actionButton.setEnabled(false);
					abortButton.setEnabled(false);
					failButton.setEnabled(false);
				}
				catch(Exception ex)
				{
					MessageDialog.openError(shell, "Error", ex.getMessage());
				}
			}
			public void mouseDoubleClick(MouseEvent e) {
			}
		});		

		
		failButton = new Button(buttonPane, SWT.PUSH);
		failButton.setText("FAIL");
		failButton.setEnabled(false);
		failButton.addMouseListener(new MouseListener() {			
			public void mouseUp(MouseEvent e) {
			}
			public void mouseDown(MouseEvent e) {
				if(selectedInstanceId ==-1)
				{
					MessageDialog.openError(shell, "Error",  "You have not selected any instance");
					return ;
				}
								
				try
				{
				
					IInformalCaregiverService icgService = RMIServiceProvider.getRMIServiceProvider().getInformalCaregiver2Service();
					icgService.abort(selectedInstanceId);
					
					MessageDialog.openInformation(parent.getShell(), "Information", "Operation aborted successfully.");
					actionButton.setEnabled(false);
					abortButton.setEnabled(false);
					failButton.setEnabled(false);
				}
				catch(Exception ex)
				{
					MessageDialog.openError(shell, "Error", ex.getMessage());
				}
			}
			public void mouseDoubleClick(MouseEvent e) {
			}
		});	
		
	}
	
	private boolean validateField(Text txtField, String fieldName)
	{
		if(txtField.getText().length() == 0){
			MessageDialog.openError(shell, "Error",  fieldName + " cannot be empty.");
			return false;
		}
		return true;
	}
	
	public void setActiveInstance(Long instanceId)
	{
		combo.setText( "Instance #" + instanceId);
		processSelection(instanceId);
	}
	
	
	private void processSelection(Long instanceId)
	{		
		this.selectedInstanceId = instanceId;
		IWorkflowEngineService wfEngineService = RMIServiceProvider.getRMIServiceProvider().getWorkflowEngineService();
		List<InstanceInfo> infos = wfEngineService.getAvailableMethods(instanceId, TASK_ID);
		
		actionButton.setEnabled(false);
		abortButton.setEnabled(false);
		failButton.setEnabled(false);
		
		for(InstanceInfo info : infos)
		{
			if(info.getAvailableMethod().equals(AbstractTask._ACTION))				
				actionButton.setEnabled(true);
			if(info.getAvailableMethod().equals(AbstractTask._ABORT))				
				abortButton.setEnabled(true);
			if(info.getAvailableMethod().equals(AbstractTask._FAIL))				
				failButton.setEnabled(true);
			
		}
		
		// find the patien by instance id and get the patient name
		IPatientRegistrationService patientService = RMIServiceProvider.getRMIServiceProvider().getPatientRegistrationService();
		PatientDTO patient = patientService.findPatientInfoByInstanceId(instanceId);
		txtPatientName.setText(patient.getLastName() + ", " + patient.getFirstName());
		txtPatientId.setText(patient.getId()+"");
		
		
	}
	
	private void populateActiveInstances()
	{
		IWorkflowEngineService wfEngineService = RMIServiceProvider.getRMIServiceProvider().getWorkflowEngineService();
		activeInstances = wfEngineService.getAllActiveInstances();
		if(activeInstances == null)
			activeInstances = new ArrayList<WfInstance>();
	}
	private WfInstance getInstanceByText(String instanceText)
	{
		for(WfInstance ins : activeInstances)
		{
			String name = "Instance #" + ins.getId();
			if(name.equals(instanceText))
				return ins;
		}
		return null;
	}

	public void setFocus() {
	}
}
