package clientapp.view;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;

import pc.Overall.service.interfaces.IPatientRegistrationService;
import pc.Overall.service.interfaces.IPhysicianService;
import pc.Overall.service.interfaces.IReceiveReferralService;
import pc.entity.PatientDTO;
import pc.entity.PhysicianDTO;
import pc.entity.ReferralInfoDTO;

import clientapp.IInstanceSelectionListener;

import ca.stfx.logic.cwf.engine.DateUtil;
import ca.stfx.logic.cwf.engine.base.AbstractTask;
import ca.stfx.logic.cwf.engine.base.entity.InstanceInfo;
import ca.stfx.logic.cwf.engine.base.entity.WfInstance;
import ca.stfx.logic.cwf.engine.service.IWorkflowEngineService;

import communicator.RMIServiceProvider;

public class PatientRegistrationView extends ViewPart implements IInstanceSelectionListener{
		
	public static final String ID = "clientapp.PatientRegistrationView";
	public static final String TASK_ID = "Overall_Intake";
	public static final String TASK_NAME = "Intake";
	public static int instanceNumber = 0;
	private List<WfInstance> activeInstances;
	
	private Button button;	
	private Shell shell;
	
	private Text txtLastName;
	private Text txtFirstName;
	private Text txtBirthDate;
	private Text txtAddress1;
	private Text txtAddress2;
	private Text txtZip;
	private Text txtPhone;
	private Combo countryCombo;
	private Combo locationCombo;	
	private Text txtSpecialComment;
	
	private Combo combo;
		
	private long selectedInstanceId = 0;
	
	public void createPartControl(final Composite parent) {
		this.shell = parent.getShell();		
		selectedInstanceId = -1;
		Composite top = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		top.setLayout(layout);
		// top banner
		Composite banner = new Composite(top, SWT.NONE);
		banner.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL, GridData.VERTICAL_ALIGN_BEGINNING, true, false));
		layout = new GridLayout();
		layout.marginHeight = 5;
		layout.marginWidth = 10;
		layout.numColumns = 1;
		banner.setLayout(layout);
		
		// setup bold font
		Font boldFont = JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT);    
		
		Label l = new Label(banner, SWT.WRAP);
		l.setText("Physician Registration. Enter new physician information to the system.");
		l.setFont(boldFont);
		
		Calendar theCal = Calendar.getInstance();
		String dateStr = "Date: " + theCal.get(Calendar.DATE) + "/" + theCal.get(Calendar.MONTH) + "/" + theCal.get(Calendar.YEAR);
		
		l = new Label(banner, SWT.WRAP);
		l.setText(dateStr);
		
			
		Composite body = new Composite(top, SWT.NONE);
		body.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL, GridData.VERTICAL_ALIGN_BEGINNING, true, false));
		layout = new GridLayout();
		layout.marginHeight = 5;
		layout.marginWidth = 10;
		layout.numColumns = 2;
		body.setLayout(layout);
		
		
		populateActiveInstances();
		
		l = new Label(body, SWT.WRAP);
		l.setText("Active Instance:");
		combo = new Combo(body, SWT.DROP_DOWN | SWT.BORDER | SWT.READ_ONLY);
		for(WfInstance instance : activeInstances)
		{
			combo.add("Instance #" + instance.getId());
		}
		
	    combo.addSelectionListener(new SelectionListener() {
			
			public void widgetSelected(SelectionEvent e) {
				if(e == null)
					return;			
				WfInstance selectedInstance = getInstanceByText(((Combo) e.getSource()).getText());
				processSelection(selectedInstance.getId());
			}
			
			public void widgetDefaultSelected(SelectionEvent e) {
				// TODO Auto-generated method stub
				
			}
		});
		
		l = new Label(body, SWT.WRAP);
		l.setText("Last Name:");
		txtLastName = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtLastName.setText("");
		txtLastName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		l = new Label(body, SWT.WRAP);
		l.setText("First Name:");
		txtFirstName = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtFirstName.setText("");
		txtFirstName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		l = new Label(body, SWT.WRAP);
		l.setText("BirthDate:");
		txtBirthDate = new Text(body, SWT.MULTI | SWT.WRAP);
		txtBirthDate.setText("");
		txtBirthDate.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Address1:");
		txtAddress1 = new Text(body, SWT.MULTI | SWT.WRAP);
		txtAddress1.setText("");
		txtAddress1.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		l = new Label(body, SWT.WRAP);
		l.setText("Address2:");
		txtAddress2 = new Text(body, SWT.MULTI | SWT.WRAP);
		txtAddress2.setText("");
		txtAddress2.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Zip:");
		txtZip = new Text(body, SWT.MULTI | SWT.WRAP);
		txtZip.setText("");
		txtZip.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Phone Number:");
		txtPhone = new Text(body, SWT.MULTI | SWT.WRAP);
		txtPhone.setText("");
		txtPhone.setLayoutData(new GridData(GridData.FILL_BOTH));
		
				
		l = new Label(body, SWT.WRAP);
		l.setText("Country:");
		countryCombo = new Combo(body, SWT.DROP_DOWN | SWT.BORDER | SWT.READ_ONLY);
		countryCombo.add("Canada" );
		countryCombo.add("USA" );
		countryCombo.add("Other");
		countryCombo.setText("Canada");

		l = new Label(body, SWT.WRAP);
		l.setText("Location:");
		locationCombo = new Combo(body, SWT.DROP_DOWN | SWT.BORDER | SWT.READ_ONLY);
		locationCombo.add("Home" );
		locationCombo.add("Hospital" );
		locationCombo.setText("Home");
		
		l = new Label(top, SWT.WRAP);
		l.setText("Comment:");
		txtSpecialComment = new Text(top, SWT.MULTI | SWT.WRAP);
		txtSpecialComment.setText("");
		txtSpecialComment.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		button = new Button(top, SWT.PUSH);
		button.setText("ADD");
		
		button.addMouseListener(new MouseListener() {			
			public void mouseUp(MouseEvent e) {
			}
			public void mouseDown(MouseEvent e) {
				// invoke the server
				if(selectedInstanceId ==-1)
				{
					MessageDialog.openError(shell, "Error",  "You have not selected any instance");
					return ;
				}
				if( validateField(txtLastName, "Last Name") == false)
					return;
				if( validateField(txtFirstName, "First Name") == false)
					return;				
				if( validateField(txtPhone, "Phone Number") == false)
					return;
				if( validateField(txtBirthDate, "Birth Date") == false)
					return;
				
				PatientDTO patient = new PatientDTO();
				patient.setInstanceId(selectedInstanceId);
				patient.setAddress1(txtAddress1.getText());
				patient.setAddress2(txtAddress2.getText());
				patient.setBirthDate(txtBirthDate.getText());
				patient.setCreationDate(DateUtil.getCurrentDate());				
				patient.setFirstName(txtFirstName.getText());
				patient.setLastName(txtLastName.getText());
				patient.setLocation(getLocation(locationCombo.getText()));		
				patient.setCountry(countryCombo.getText());
				patient.setPhoneNumber(txtPhone.getText());
				patient.setSpecialComment(txtSpecialComment.getText());
				patient.setZip(txtZip.getText());
								
				IPatientRegistrationService patientService = RMIServiceProvider.getRMIServiceProvider().getPatientRegistrationService();
				try
				{
					patient = patientService.savePatientInfo(patient);
					MessageDialog.openInformation(parent.getShell(), "Information", "Patient Id " + patient.getId() + " created successfully.");					
				}
				catch(Exception ex)
				{
					MessageDialog.openError(shell, "Error", ex.getMessage());
				}
			}
			public void mouseDoubleClick(MouseEvent e) {
			}
		});

	}
	
	private Integer getLocation(String type)
	{
		if(type.equals("Home"))
			return PatientDTO.LOCATION_HOME;
		else
			return PatientDTO.LOCATION_HOSPITAL;
	}
	
	private boolean validateField(Text txtField, String fieldName)
	{
		if(txtField.getText().length() == 0){
			MessageDialog.openError(shell, "Error",  fieldName + " cannot be empty.");
			return false;
		}
		return true;
	}
	
	public void setActiveInstance(Long instanceId)
	{
		combo.setText( "Instance #" + instanceId);
		processSelection(instanceId);
	}
	
	
	private void processSelection(Long instanceId)
	{		
		this.selectedInstanceId = instanceId;
		IWorkflowEngineService wfEngineService = RMIServiceProvider.getRMIServiceProvider().getWorkflowEngineService();
		List<InstanceInfo> infos = wfEngineService.getAvailableMethods(instanceId, TASK_ID);
		button.setEnabled(false);
		
		for(InstanceInfo info : infos)
		{
			if(info.getAvailableMethod().equals(AbstractTask._ACTION)){
				button.setEnabled(true);
				return;
			}
		}
	}
	
	private void populateActiveInstances()
	{
		IWorkflowEngineService wfEngineService = RMIServiceProvider.getRMIServiceProvider().getWorkflowEngineService();
		activeInstances = wfEngineService.getAllActiveInstances();
		if(activeInstances == null)
			activeInstances = new ArrayList<WfInstance>();
	}
	
	private WfInstance getInstanceByText(String instanceText)
	{
		for(WfInstance ins : activeInstances)
		{
			String name = "Instance #" + ins.getId();
			if(name.equals(instanceText))
				return ins;
		}
		return null;
	}
	public void setFocus() {
	}
}
