package clientapp.view;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;

import pc.Overall.service.interfaces.IPhysicianService;
import pc.Overall.service.interfaces.IReceiveReferralService;
import pc.entity.PhysicianDTO;
import pc.entity.ReferralInfoDTO;

import clientapp.IInstanceSelectionListener;

import ca.stfx.logic.cwf.engine.DateUtil;
import ca.stfx.logic.cwf.engine.base.AbstractTask;
import ca.stfx.logic.cwf.engine.base.entity.InstanceInfo;
import ca.stfx.logic.cwf.engine.base.entity.WfInstance;
import ca.stfx.logic.cwf.engine.service.IWorkflowEngineService;

import communicator.RMIServiceProvider;

public class PhysicianRegistrationView extends ViewPart {
		
	public static final String ID = "clientapp.PhysicianRegistrationView";
	public static int instanceNumber = 0;
	
	private Button button;	
	private Shell shell;
	
	private Text txtLastName;
	private Text txtFirstName;
	private Text txtDesignation;
	private Text txtDepartment;
	private Text txtAddress1;
	private Text txtAddress2;
	private Text txtBirthDate;
	private Text txtPhone;
	private Text txtZip;
	private Combo combo;
		
	public void createPartControl(final Composite parent) {
		this.shell = parent.getShell();		
		Composite top = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		top.setLayout(layout);
		// top banner
		Composite banner = new Composite(top, SWT.NONE);
		banner.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL, GridData.VERTICAL_ALIGN_BEGINNING, true, false));
		layout = new GridLayout();
		layout.marginHeight = 5;
		layout.marginWidth = 10;
		layout.numColumns = 1;
		banner.setLayout(layout);
		
		// setup bold font
		Font boldFont = JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT);    
		
		Label l = new Label(banner, SWT.WRAP);
		l.setText("Physician Registration. Enter new physician information to the system.");
		l.setFont(boldFont);
		
		Calendar theCal = Calendar.getInstance();
		String dateStr = "Date: " + theCal.get(Calendar.DATE) + "/" + theCal.get(Calendar.MONTH) + "/" + theCal.get(Calendar.YEAR);
		
		l = new Label(banner, SWT.WRAP);
		l.setText(dateStr);
		
			
		Composite body = new Composite(top, SWT.NONE);
		body.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL, GridData.VERTICAL_ALIGN_BEGINNING, true, false));
		layout = new GridLayout();
		layout.marginHeight = 5;
		layout.marginWidth = 10;
		layout.numColumns = 2;
		body.setLayout(layout);
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Last Name:");
		txtLastName = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtLastName.setText("");
		txtLastName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		l = new Label(body, SWT.WRAP);
		l.setText("First Name:");
		txtFirstName = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtFirstName.setText("");
		txtFirstName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		l = new Label(body, SWT.WRAP);
		l.setText("Designation:");
		txtDesignation = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtDesignation.setText("");
		txtDesignation.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		l = new Label(body, SWT.WRAP);
		l.setText("Department:");
		txtDepartment = new Text(body, SWT.SINGLE | SWT.WRAP);
		txtDepartment.setText("");
		txtDepartment.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		l = new Label(body, SWT.WRAP);
		l.setText("Address1:");
		txtAddress1 = new Text(body, SWT.MULTI | SWT.WRAP);
		txtAddress1.setText("");
		txtAddress1.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		l = new Label(body, SWT.WRAP);
		l.setText("Address2:");
		txtAddress2 = new Text(body, SWT.MULTI | SWT.WRAP);
		txtAddress2.setText("");
		txtAddress2.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		l = new Label(body, SWT.WRAP);
		l.setText("BirthDate:");
		txtBirthDate = new Text(body, SWT.MULTI | SWT.WRAP);
		txtBirthDate.setText("");
		txtBirthDate.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		l = new Label(body, SWT.WRAP);
		l.setText("Phone Number:");
		txtPhone = new Text(body, SWT.MULTI | SWT.WRAP);
		txtPhone.setText("");
		txtPhone.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		
		l = new Label(body, SWT.WRAP);
		l.setText("Zip:");
		txtZip = new Text(body, SWT.MULTI | SWT.WRAP);
		txtZip.setText("");
		txtZip.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		l = new Label(body, SWT.WRAP);
		l.setText("Type:");
		combo = new Combo(body, SWT.DROP_DOWN | SWT.BORDER | SWT.READ_ONLY);
			combo.add("Family Physician" );
			combo.add("PC Physician" );
		combo.setText("PC Physician");
		
		
		button = new Button(top, SWT.PUSH);
		button.setText("ADD");
		
		button.addMouseListener(new MouseListener() {			
			public void mouseUp(MouseEvent e) {
			}
			public void mouseDown(MouseEvent e) {
				// invoke the server
				if( validateField(txtLastName, "Last Name") == false)
					return;
				if( validateField(txtFirstName, "First Name") == false)
					return;
				if( validateField(txtDesignation, "Designation") == false)
					return;
				if( validateField(txtPhone, "Phone Number") == false)
					return;
				
				PhysicianDTO physician = new PhysicianDTO();
				physician.setAddress1(txtAddress1.getText());
				physician.setAddress2(txtAddress2.getText());
				physician.setBirthDate(txtBirthDate.getText());
				physician.setCreationDate(DateUtil.getCurrentDate());
				physician.setDepartment(txtDepartment.getText());
				physician.setDesignation(txtDesignation.getText());
				physician.setFirstName(txtFirstName.getText());
				physician.setLastName(txtLastName.getText());
				physician.setPhoneNumber(txtPhone.getText());
				physician.setType( getPhysicianType( combo.getText() ));
				physician.setZip(txtZip.getText());
				
				
				IPhysicianService physicianService = RMIServiceProvider.getRMIServiceProvider().getPhysicianService();
				try
				{
					physician = physicianService.savePhysician(physician);
					MessageDialog.openInformation(parent.getShell(), "Information", "Physician Id " + physician.getId() + " created successfully.");					
				}
				catch(Exception ex)
				{
					MessageDialog.openError(shell, "Error", ex.getMessage());
				}
			}
			public void mouseDoubleClick(MouseEvent e) {
			}
		});

	}
	
	private Integer getPhysicianType(String type)
	{
		if(type.equals("Family Physician"))
			return PhysicianDTO.TYPE_FAMILY_PH;
		else
			return PhysicianDTO.TYPE_PC_PH;
	}
	
	private boolean validateField(Text txtField, String fieldName)
	{
		if(txtField.getText().length() == 0){
			MessageDialog.openError(shell, "Error",  fieldName + " cannot be empty.");
			return false;
		}
		return true;
	}
	
	
	public void setFocus() {
	}
}
