package clientapp.view;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;

import clientapp.IInstanceSelectionListener;

import communicator.RMIServiceProvider;

import ca.stfx.logic.cwf.engine.DateUtil;
import ca.stfx.logic.cwf.engine.base.entity.InstanceInfo;
import ca.stfx.logic.cwf.engine.base.entity.WfInstance;
import ca.stfx.logic.cwf.engine.service.IWorkflowEngineService;
import ca.stfx.logic.cwf.mc.base.Util;

public class WorklistView extends ViewPart {

	public static final String ID = "clientapp.WorklistView";
	public static int instanceNumber = 0;
	
	private List<WfInstance> activeInstances;
	private List<InstanceInfo> availableMethods;
	private Table table;
	private Shell shell;
	Composite top; 
	
	public void createPartControl(final Composite parent) {
		this.top = new Composite(parent, SWT.NONE);
		this.shell = parent.getShell();
		GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		top.setLayout(layout);
		// top banner
		Composite banner = new Composite(top, SWT.NONE);
		banner.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL, GridData.VERTICAL_ALIGN_BEGINNING, true, false));
		layout = new GridLayout();
		layout.marginHeight = 5;
		layout.marginWidth = 10;
		layout.numColumns = 1;
		banner.setLayout(layout);
		
		// setup bold font
		Font boldFont = JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT);    
		
		Label l = new Label(banner, SWT.WRAP);
		l.setText("Select an instance from the drop down.\nAvailable workitems will be displayed in the list");
		l.setFont(boldFont);
						
		Composite body = new Composite(top, SWT.NONE);
		body.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL, GridData.VERTICAL_ALIGN_BEGINNING, true, false));
		layout = new GridLayout();
		layout.marginHeight = 5;
		layout.marginWidth = 10;
		layout.numColumns = 2;
		body.setLayout(layout);
		
		populateActiveInstances();
		
		l = new Label(body, SWT.WRAP);
		l.setText("Active Instance:");
		Combo combo = new Combo(body, SWT.DROP_DOWN | SWT.BORDER | SWT.READ_ONLY);
		for(WfInstance instance : activeInstances)
		{
			combo.add("Instance #" + instance.getId());
		}
		
	    combo.addSelectionListener(new SelectionListener() {
			
			public void widgetSelected(SelectionEvent e) {
				if(e == null)
					return;				
				WfInstance selectedInstance = getInstanceByText(((Combo) e.getSource()).getText());
				//populate the table now
				fillTable(table, selectedInstance);
			}
			
			public void widgetDefaultSelected(SelectionEvent e) {
				// TODO Auto-generated method stub
				
			}
		});
	    
	    
	    
		table = new Table(top, SWT.NONE);
	    table.setHeaderVisible(true);
	    table.setLinesVisible(true);
	    TableColumn[] column = new TableColumn[4];
	    column[0] = new TableColumn(table, SWT.NONE);
	    column[0].setText("Index");
	    
	    column[1] = new TableColumn(table, SWT.NONE);
	    column[1].setText("Task Name");
	    
	    column[2] = new TableColumn(table, SWT.NONE);
	    column[2].setText("Operation");
	    
	    column[3] = new TableColumn(table, SWT.RIGHT);
	    column[3].setText("Operation Type");
	    
	    for (int i = 0, n = column.length; i < n; i++) {
	      column[i].pack();
	    }
	    table.setLayoutData(new GridData(GridData.FILL_BOTH));
	    
	    table.addListener(SWT.Selection, new Listener() {
	        public void handleEvent(Event e) {
	          TableItem[] selection = table.getSelection();
	          if(selection == null || selection.length == 0 )
	        	  return;
	          String indexNo = selection[0].getText();
	          System.out.println("Selected table entry : " + indexNo);
	          int index = Integer.parseInt(indexNo);
	          index--;
	          InstanceInfo selectedEntry = availableMethods.get(index);
	          try
				{
	        	  System.out.println("Task id : " + selectedEntry.getTaskId());
	        	  if(selectedEntry.getTaskId().equals(ReceiveReferralView.TASK_ID))
	        	  {						
						IInstanceSelectionListener receiveReferralView = (IInstanceSelectionListener) getSite().getWorkbenchWindow().getActivePage().showView(ReceiveReferralView.ID, Integer.toString(ReceiveReferralView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);							
						receiveReferralView.setActiveInstance(selectedEntry.getInstanceId());							
	        	  }
	        	  else if(selectedEntry.getTaskId().equals(AppointmentView.TASK_ID))
	        	  {						
						IInstanceSelectionListener appointmentView = (IInstanceSelectionListener) getSite().getWorkbenchWindow().getActivePage().showView(AppointmentView.ID, Integer.toString(AppointmentView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);							
						appointmentView.setActiveInstance(selectedEntry.getInstanceId());							
	        	  }
	        	  else if(selectedEntry.getTaskId().equals(ConsultView.TASK_ID))
	        	  {						
						IInstanceSelectionListener consultView = (IInstanceSelectionListener) getSite().getWorkbenchWindow().getActivePage().showView(ConsultView.ID, Integer.toString(ConsultView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);							
						consultView.setActiveInstance(selectedEntry.getInstanceId());
	        	  }
	        	  else if(selectedEntry.getTaskId().equals(PatientRegistrationView.TASK_ID))
	        	  {						
						IInstanceSelectionListener intakeView = (IInstanceSelectionListener) getSite().getWorkbenchWindow().getActivePage().showView(PatientRegistrationView.ID, Integer.toString(PatientRegistrationView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);							
						intakeView.setActiveInstance(selectedEntry.getInstanceId());							
	        	  }
	        	  else if(selectedEntry.getTaskId().equals(FindHomeServiceView.TASK_ID))
	        	  {						
						IInstanceSelectionListener findHSView = (IInstanceSelectionListener) getSite().getWorkbenchWindow().getActivePage().showView(FindHomeServiceView.ID, Integer.toString(FindHomeServiceView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);							
						findHSView.setActiveInstance(selectedEntry.getInstanceId());							
	        	  }
	        	  else if(selectedEntry.getTaskId().equals(MoveToHospitalView.TASK_ID))
	        	  {						
						IInstanceSelectionListener moveView = (IInstanceSelectionListener) getSite().getWorkbenchWindow().getActivePage().showView(MoveToHospitalView.ID, Integer.toString(MoveToHospitalView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);							
						moveView.setActiveInstance(selectedEntry.getInstanceId());							
	        	  }
	        	  else if(selectedEntry.getTaskId().equals(InformalCaregiverView.TASK_ID))
	        	  {						
						IInstanceSelectionListener icgView = (IInstanceSelectionListener) getSite().getWorkbenchWindow().getActivePage().showView(InformalCaregiverView.ID, Integer.toString(InformalCaregiverView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);							
						icgView.setActiveInstance(selectedEntry.getInstanceId());							
	        	  }
	        	  else if(selectedEntry.getTaskId().equals(InformalCaregiver2View.TASK_ID))
	        	  {						
						IInstanceSelectionListener icgView = (IInstanceSelectionListener) getSite().getWorkbenchWindow().getActivePage().showView(InformalCaregiver2View.ID, Integer.toString(InformalCaregiver2View.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);							
						icgView.setActiveInstance(selectedEntry.getInstanceId());							
	        	  }
	        	  else if(selectedEntry.getTaskId().equals(FamilyPhysicianView.TASK_ID))
	        	  {						
						IInstanceSelectionListener fphView = (IInstanceSelectionListener) getSite().getWorkbenchWindow().getActivePage().showView(FamilyPhysicianView.ID, Integer.toString(FamilyPhysicianView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);							
						fphView.setActiveInstance(selectedEntry.getInstanceId());							
	        	  }
	        	  else if(selectedEntry.getTaskId().equals(PCPhysicianView.TASK_ID))
	        	  {						
						IInstanceSelectionListener pphView = (IInstanceSelectionListener) getSite().getWorkbenchWindow().getActivePage().showView(PCPhysicianView.ID, Integer.toString(PCPhysicianView.instanceNumber++), IWorkbenchPage.VIEW_ACTIVATE);							
						pphView.setActiveInstance(selectedEntry.getInstanceId());							
	        	  }
	          } catch (PartInitException pe) {
					//MessageDialog.openError(parent.getShell(), "Error", "Error opening view:" + e.getMessage());
				}
	        }
	      });
	    
	}

	
	 private void fillTable(final Table table, WfInstance selectedInstance) {
		    table.setRedraw(false);
		   
		    // get data from server
		    IWorkflowEngineService wfEngineService = RMIServiceProvider.getRMIServiceProvider().getWorkflowEngineService();
		    availableMethods = wfEngineService.getAvailableMethods(selectedInstance);
		    if(availableMethods == null)
		    	return;
		    		    
		    table.removeAll();
		    
		    for(int i = 0; i < availableMethods.size(); ++i){
		      InstanceInfo theInsInfo = availableMethods.get(i);
		      TableItem item = new TableItem(table, SWT.NONE);
		      int c = 0;
		      item.setText(c++, ""+(i+1) );
		      item.setText(c++, theInsInfo.getTaskId());
		      item.setText(c++, "" + theInsInfo.getAvailableMethod());
		      item.setText(c++, theInsInfo.getActor());
		            
		    }
		    table.setLayoutData(new GridData(GridData.FILL_BOTH));
		    table.setRedraw(true);		    		    
		  }
	 
	private void populateActiveInstances()
	{
		IWorkflowEngineService wfEngineService = RMIServiceProvider.getRMIServiceProvider().getWorkflowEngineService();
		activeInstances = wfEngineService.getAllActiveInstances();
		if(activeInstances == null)
			activeInstances = new ArrayList<WfInstance>();
	}
	
	private WfInstance getInstanceByText(String instanceText)
	{
		for(WfInstance ins : activeInstances)
		{
			String name = "Instance #" + ins.getId();
			if(name.equals(instanceText))
				return ins;
		}
		return null;
	}
	
	public void setFocus() {
	}
}
